@@ -1,25 +1,22 @@
 from libspn_keras.backprop_mode import BackpropMode
-from libspn_keras.dimension_permutation import DimensionPermutation
 from libspn_keras.logspace import logspace_wrapper_initializer
-from libspn_keras.models import SpatialSumProductNetwork, DenseSumProductNetwork
-from libspn_keras.normalizationaxes import NormalizationAxes
+from libspn_keras.utils.generative_learning_em import GenerativeLearningEM
 from libspn_keras import optimizers
 from libspn_keras import metrics
 from libspn_keras import losses
 from libspn_keras import layers
 from libspn_keras import constraints
+from libspn_keras import initializers
 from libspn_keras.region import RegionNode
 from libspn_keras.region import RegionVariable
 from libspn_keras.region import region_graph_to_dense_spn
 from libspn_keras.visualize import visualize_dense_spn
+from libspn_keras import utils
+from libspn_keras import models
 
 __all__ = [
     'BackpropMode',
-    'DimensionPermutation',
     'logspace_wrapper_initializer',
-    'SpatialSumProductNetwork',
-    'DenseSumProductNetwork',
-    'NormalizationAxes',
     'optimizers',
     'metrics',
     'losses',
@@ -28,5 +25,9 @@ __all__ = [
     'RegionNode',
     'RegionVariable',
     'region_graph_to_dense_spn',
-    'visualize_dense_spn'
+    'visualize_dense_spn',
+    'utils',
+    'initializers',
+    'GenerativeLearningEM',
+    'models'
 ]
