@@ -12,3 +12,7 @@ class BackpropMode:
     HARD_EM = "hard_em"
     HARD_EM_UNWEIGHTED = "hard_em_unweighted"
     EM = "em"
+
+
+def infer_logspace_accumulators(backprop_mode):
+    return backprop_mode == BackpropMode.GRADIENT
