@@ -19,19 +19,18 @@ class FlatToRegions(keras.layers.Layer):
     def call(self, inputs):
         if len(inputs.shape) == 2:
             inputs = tf.expand_dims(inputs, axis=-1)
-        transposed = tf.transpose(inputs, (1, 0, 2))
         with_decomps = tf.tile(
-            tf.expand_dims(transposed, axis=1), multiples=(1, self.num_decomps, 1, 1))
+            tf.expand_dims(inputs, axis=2), multiples=(1, 1, self.num_decomps, 1))
         return with_decomps
 
     def compute_output_shape(self, input_shape):
 
         if len(input_shape) == 2:
             num_batch, num_vars = input_shape
-            return [num_vars, self.num_decomps, num_batch, 1]
+            var_dimensionality = 1
         else:
             num_batch, num_vars, var_dimensionality = input_shape
-            return [num_vars, self.num_decomps, num_batch, var_dimensionality]
+        return [num_batch, num_vars, self.num_decomps, var_dimensionality]
 
     def get_config(self):
         config = dict(
