@@ -3,8 +3,9 @@ import abc
 from libspn_keras.layers.base_leaf import BaseLeaf
 import tensorflow_probability as tfp
 from tensorflow import initializers
+import tensorflow as tf
 
-from libspn_keras.math.soft_em_grads import LocationScaleEMGradWrapper
+from libspn_keras.math.soft_em_grads import LocationScaleEMGradWrapper, LocationEMGradWrapper
 
 
 class LocationScaleLeafBase(BaseLeaf):
@@ -20,56 +21,84 @@ class LocationScaleLeafBase(BaseLeaf):
     """
     def __init__(
         self, num_components, location_initializer=None,
-        location_trainable=True, scale_initializer=None,
+        location_trainable=True, scale_initializer=None, scale_trainable=None,
         accumulator_initializer=None, use_accumulators=False, **kwargs
     ):
         super(LocationScaleLeafBase, self).__init__(num_components=num_components, **kwargs)
         self.location_initializer = location_initializer or initializers.TruncatedNormal(stddev=1.0)
         self.location_trainable = location_trainable
         self.scale_initializer = scale_initializer or initializers.Ones()
+        self.scale_trainable = scale_trainable
         self.accumulator_initializer = accumulator_initializer or initializers.Ones()
         self.use_accumulators = use_accumulators
-        self._distribution = self._num_scopes = self._num_decomps = None
+        self._num_scopes = self._num_decomps = None
 
     def _build_distribution(self, shape):
         if self.use_accumulators:
-            denom_accumulator, num_accumulator, scale = \
-                self._create_loc_accumulators_and_scale(shape)
-            loc = num_accumulator / denom_accumulator
-            dist = self._build_distribution_from_loc_and_scale(loc=loc, scale=scale)
-            return LocationScaleEMGradWrapper(dist, denom_accumulator, num_accumulator)
+            self._create_loc_scale_accumulators(shape)
+            self._get_distribution = self._get_distribution_from_accumulators
         else:
-            loc, scale = self._create_location_and_scale_variable(shape)
-            return self._build_distribution_from_loc_and_scale(loc=loc, scale=scale)
+            self._create_loc_scale_vars(shape)
+            self._get_distribution = self._get_distribution_from_vars
 
-    def _create_loc_accumulators_and_scale(self, shape):
-        denom_accumulator = self.add_weight(
-            name="denom_accumulator", shape=shape, initializer=self.accumulator_initializer,
+    def _get_distribution_from_vars(self):
+        return self._build_distribution_from_loc_and_scale(loc=self.loc, scale=self.scale)
+
+    def _get_distribution_from_accumulators(self):
+        loc = self.first_order_moment_num_accum / self.first_order_moment_denom_accum
+        if self.scale_trainable:
+            scale = tf.sqrt(self.second_order_moment_num_accum / self.second_order_moment_denom_accum - tf.square(loc))
+            dist = self._build_distribution_from_loc_and_scale(loc=loc, scale=scale)
+            return LocationScaleEMGradWrapper(
+                dist, self.first_order_moment_denom_accum, self.first_order_moment_num_accum,
+                self.second_order_moment_denom_accum, self.second_order_moment_num_accum)
+        else:
+            dist = self._build_distribution_from_loc_and_scale(loc=loc, scale=self.scale)
+            return LocationEMGradWrapper(dist, self.first_order_moment_denom_accum, self.first_order_moment_num_accum)
+
+    def _create_loc_scale_accumulators(self, shape):
+        self.first_order_moment_denom_accum = self.add_weight(
+            name="first_order_moment_denom_accum", shape=shape, initializer=self.accumulator_initializer,
             trainable=self.location_trainable)
-        num_accumulator = self.add_weight(
-            name="num_accumulator", shape=shape, initializer=self.location_initializer,
+        self.first_order_moment_num_accum = self.add_weight(
+            name="first_order_moment_num_accum", shape=shape, initializer=self.location_initializer,
             trainable=self.location_trainable)
-        scale = self.add_weight(
-            name="scale", shape=shape, initializer=self.scale_initializer, trainable=False)
-        return denom_accumulator, num_accumulator, scale
+        self.first_order_moment_num_accum.assign(
+            self.first_order_moment_num_accum * self.first_order_moment_denom_accum)
+
+        if self.scale_trainable:
+            self.second_order_moment_denom_accum = self.add_weight(
+                name="second_order_moment_denom_accum", shape=shape, initializer=self.accumulator_initializer,
+                trainable=True)
+            self.second_order_moment_num_accum = self.add_weight(
+                name="second_order_moment_num_accum", shape=shape, initializer=self.scale_initializer,
+                trainable=True)
+            loc = self.first_order_moment_num_accum / self.first_order_moment_denom_accum
+            self.second_order_moment_num_accum.assign(
+                (tf.square(self.second_order_moment_num_accum) + tf.square(loc)) * self.second_order_moment_denom_accum
+            )
+        else:
+            self.scale = self.add_weight(
+                name="scale", shape=shape, initializer=self.scale_initializer, trainable=False
+            )
 
     @abc.abstractmethod
     def _build_distribution_from_loc_and_scale(self, loc, scale):
         """ Implement in descendant classes"""
 
-    def _create_location_and_scale_variable(self, shape):
-        loc = self.add_weight(
+    def _create_loc_scale_vars(self, shape):
+        self.loc = self.add_weight(
             name="location", shape=shape, initializer=self.location_initializer,
             trainable=self.location_trainable)
-        scale = self.add_weight(
+        self.scale = self.add_weight(
             name="scale", shape=shape, initializer=self.scale_initializer, trainable=False)
-        return loc, scale
 
     def get_config(self):
         config = dict(
             scale_initializer=initializers.serialize(self.scale_initializer),
             location_initializer=initializers.serialize(self.location_initializer),
-            accumulator_intiailizer=initializers.serialize(self.accumulator_initializer),
+            accumulator_initializer=initializers.serialize(self.accumulator_initializer),
+            scale_trainable=self.scale_trainable,
             use_accumulators=self.use_accumulators,
             location_trainable=self.location_trainable,
         )
@@ -77,7 +106,7 @@ class LocationScaleLeafBase(BaseLeaf):
         return dict(list(base_config.items()) + list(config.items()))
 
     def get_modes(self):
-        return self._distribution.mode()
+        return self._get_distribution().mode()
 
 
 class NormalLeaf(LocationScaleLeafBase):
@@ -108,9 +137,19 @@ class CauchyLeaf(LocationScaleLeafBase):
         location_initializer: Initializer for location variable
         location_trainable: Boolean that indicates whether location is trainable
         scale_initializer: Initializer for scale variable
-        scale_trainable: Boolean that indicates whether scale is trainable
         **kwargs: kwargs to pass on to the keras.Layer super class
     """
+    def __init__(self, num_components, location_initializer=None, location_trainable=True, scale_initializer=None,
+                 accumulator_initializer=None, use_accumulators=False, **kwargs):
+        super().__init__(
+            num_components=num_components,
+            location_initializer=location_initializer,
+            location_trainable=location_trainable,
+            scale_initializer=scale_initializer,
+            scale_trainable=False,
+            accumulator_initializer=accumulator_initializer,
+            use_accumulators=use_accumulators,
+            **kwargs)
 
     def _build_distribution_from_loc_and_scale(self, loc, scale):
         return tfp.distributions.Cauchy(loc=loc, scale=scale)
@@ -126,9 +165,19 @@ class LaplaceLeaf(LocationScaleLeafBase):
         location_initializer: Initializer for location variable
         location_trainable: Boolean that indicates whether location is trainable
         scale_initializer: Initializer for scale variable
-        scale_trainable: Boolean that indicates whether scale is trainable
         **kwargs: kwargs to pass on to the keras.Layer super class
     """
+    def __init__(self, num_components, location_initializer=None, location_trainable=True, scale_initializer=None,
+                 accumulator_initializer=None, use_accumulators=False, **kwargs):
+        super().__init__(
+            num_components=num_components,
+            location_initializer=location_initializer,
+            location_trainable=location_trainable,
+            scale_initializer=scale_initializer,
+            scale_trainable=False,
+            accumulator_initializer=accumulator_initializer,
+            use_accumulators=use_accumulators,
+            **kwargs)
 
     def _build_distribution_from_loc_and_scale(self, loc, scale):
         return tfp.distributions.Laplace(loc=loc, scale=scale)
