@@ -8,7 +8,7 @@ class SpatialToRegions(keras.layers.Layer):
     scopes and decomps (which will be ``[1, 1]``).
     """
     def call(self, inputs):
-        return tf.reshape(inputs, [1, 1, -1, self._num_nodes_out])
+        return tf.reshape(inputs, [-1, 1, 1, self._num_nodes_out])
 
     def build(self, input_shape):
         num_batch, num_cells_vertical, num_cells_horizontal, num_nodes = input_shape
@@ -16,4 +16,4 @@ class SpatialToRegions(keras.layers.Layer):
 
     def compute_output_shape(self, input_shape):
         num_batch, num_cells_vertical, num_cells_horizontal, num_nodes = input_shape
-        return [1, 1, num_batch, num_cells_vertical * num_cells_horizontal * num_nodes]
\ No newline at end of file
+        return [num_batch, 1, 1, num_cells_vertical * num_cells_horizontal * num_nodes]
