@@ -1,7 +1,7 @@
 from libspn_keras.losses.negative_log_joint import NegativeLogJoint
-from libspn_keras.losses.negative_log_marginal import NegativeLogMarginal
+from libspn_keras.losses.negative_log_likelihood import NegativeLogLikelihood
 
 __all__ = [
     'NegativeLogJoint',
-    'NegativeLogMarginal'
+    'NegativeLogLikelihood'
 ]
\ No newline at end of file
